var express = require( 'express' );
var router = express.Router();
var os = require('os');
var request = require('request');
var requireNocache = require('require-nocache')(module);
var fs = require('fs-extra');
var path = require('path');

var DatabaseService = require('../database-service');

var customInterfacePathBase = '../repository/project/custom-interface/';

router.all('/custom-interface/p/:project_id/call/:interface_name', function(req, res, next) {
    // Mock getDatabaseService
    req.getDatabaseService = function() {
        var dbConfiguration;
        try {
            if (!req.query._d_b_) {
                return res.status(500).send('Invalid database config.');
            }
            dbConfiguration = JSON.parse(req.query._d_b_);
        } catch (e) {
            return next(
                new Error('Invalid database setting for preview.')
            );
        }
        return new DatabaseService(dbConfiguration);
    };
    
    var serverVars = req.query._s_v_;
    try {
        serverVars = JSON.parse(decodeURIComponent(serverVars));
    } catch(e) {
        serverVars = {};
    }

    // Append Session Variables
    var vbs = req.session._variables;
    if (vbs) {
        for (var i in vbs) {
            serverVars[i.toUpperCase()] = vbs[i];
        }
    }
    // Mock getServerVariables
    req.getServerVariables = function() {
        return serverVars;
    };

    // Mock getUser
    req.getUser = function() {
        return {
            getId: function() {
                return serverVars['USER_ID'];
            },
            getName: function() {
                return serverVars['USER_NAME'];
            },
            getRoles: function() {
                return serverVars['ROLES'];
            }
        }
    };

    // Mock enhancer api
    global.Enhancer = {
        getDatabaseService: function() {
            return req.getDatabaseService();
        },
        getDatabaseConfig: function() {
            var dbConfiguration = JSON.parse(req.query._d_b_);
            return dbConfiguration;
        },
        getVariable: function(name) {
            name = name.toUpperCase();
            return serverVars[name];
        },
        // setVariable: function(name, value) {
        //     name = name.toUpperCase();
        //     serverVars[name] = value;
        // },
        setSessionVariable: function(name, value) {
            if (!req.session._variables) {
                req.session._variables = {};
            }
            req.session._variables[name.toUpperCase()] = value;
        },
        getSessionVariable: function(name) {
            if (!name) {
                return
            }
            return (req.session._variables || {})[name.toUpperCase()];
        },
        getContextRequest: function() {
            return req;
        },
        getContextResponse: function() {
            return res;
        },
        setData: function(key, value) {
            if (!key || typeof key !== 'string') {
                throw new Error('The key must be a non-empty string.');
            }
            if (!this.data) {
                this.data = {};
            }
            req.session['_mock_' + key] = value;
        },
        getData: function(key, callback) {
            if (!key || typeof key !== 'string') {
                throw new Error('The key must be a non-empty string.');
            }
            if (typeof callback !== 'function') {
                throw new Error('Can not get data whithout callback function');
            }
            callback(null, req.session['_mock_' + key]);
        },
        getUserFileDir: function() {
            return path.join(os.homedir(), 'enhancer/data');
        },
        resolveUserFilePath: function(url) {
            if (!url || typeof url !== 'string') {
                return null;
            }
            var p = url.split(/\/file\//)[1];
            if (!p) {
                return null;
            }
            var baseDir = this.getUserFileDir();
            return baseDir + '/' + p;
        }
    };

    res.setHeader('Access-Control-Allow-Origin', '*');

    var pid = req.params.project_id;
    var iname = req.params.interface_name;

    var projectHost = 'https://workbench.enhancer.io';
    if (/previewer\.enhancer\.cc/.test(req.headers.referer)) {
        projectHost = 'http://workbench.enhancer.cc';
    }
    var url = projectHost + '/proj/' + pid + '/httpinterface/name/' + iname + '/' + req.method;

    var that = this;

    // Get interface definition.
    request({
        url: url,
        headers: {
            'Cookie': 'token=' + req.query._u_t_k_
        }
    }, function(err, r, body) {
        if (err) {
            console.error(err);
            res.status(500)
                .send('Failed to get interface by name \'' + iname + '\'. Reason: ' + err.message);
            return ;
        }
        if (r.statusCode !== 200) {
            console.error(r.statusCode, r.body);
            res.status(500)
                .send('Failed to get interface \'' + iname + '\'. statusCode: ' + r.statusCode);
            return;
        }
        var data, interface;
        try {
            data = JSON.parse(body);
            if (!data.success || !data.result) {
                return res.status(500).send('Failed to get interface \'' + iname + '\'. responseText: ' + body);
            }
        } catch(err) {
            console.log(err);
            return res.status(403).send('Not authorized for ' + iname);
        }

        interface = data.result.interface.replace(/@\w+(\.\w+)*@/g, function(s) {
            return 'Enhancer.getVariable("' + s.split(/@/g)[1].toUpperCase() + '")';
        });

        var file = path.resolve(__dirname, customInterfacePathBase + iname);
        fs.ensureFileSync(file);
        fs.writeFileSync(file, interface);

        var f = requireNocache(customInterfacePathBase + iname);

        if (typeof f !== 'function') {
            return res.status(500)
                .send('[Interface Error] No function module is exported.');
        }
 
        var isCompleted = false;
        res.on('finish', function(e) {
            isCompleted = true;
        });
        setTimeout(function() {
            if (!isCompleted) {
                isCompleted = true;
                res.status(500)
                    .end('[Timeout Error] Interface response timeout, please check if it responses correctly.');
            }
        }, 5000);
        try {
            f.apply(that, [req, res, Enhancer]);
        } catch(err) {
            console.log(err);
            isCompleted = true;
            return res.status(500)
                .send('[Interface Calling Error] Error occured when calling interface: '
                    + err.stack);
        }
    });

});

module.exports = router;